/* stmt.c
	P. Chongstitvatana		25 Aug 97

	start project RZ							3 Apr 2001
	incorporate dcl.c into stmt.c
	  and change to one pass					3 Jan 2002
	modify to untangle parsing parameters
	  in function call and use string stack		17 Oct 2010
	SQL where clause parser  				14 Feb 2016 (Valentine!)
*/

#include "compile.h"

#define MAXYSTK  	2000
#define MAXZSTK  	100
#define MAXSTRBUF 	5000

extern int lv;							// symtab.c

int pv;
char strbuf[MAXSTRBUF];					//  string buffer
int freestr = 0;						//  index to free string

//PRIVATE int DS = DSBASE;				// index to data segment
PRIVATE int ystack[MAXYSTK], ysp = 0;  	// 	parser stack
PRIVATE int zstack[MAXZSTK], zsp = 0;   //  operator stack
PRIVATE string strstk[MAXSTK];			//  string stack
PRIVATE int strstkp = 0;				//  string stack pointer

void expect( int tktype, char *mess ){
	if( tok != tktype ) seterror(mess);
}

void seterror( char *mess ){
	printf("line %d near '%s' : %s\n",line,tokstring,mess);
//  dumpSym2();
	exit(-1);
}

void warning( char *nm, char *mess){
	printf("Warning: %s %s\n",nm,mess);
}

// push string
void pusht(char *nm){
	strstkp++;
	if(strstkp > MAXSTK) seterror("string stack overflow");
	strcpy(strstk[strstkp],nm);
}

char *popt(void){
	if(strstkp <= 0) seterror("string stack underflow");
	return strstk[strstkp--];
}
/*
PRIVATE void chktype(int type1, int type2) {
	if( type1 != type2 ) seterror("identifier type mismatch");
}

PRIVATE int installFunc(char *name, int pv){
	int a, found, type, ref, arg;
	a = installGlobal(name,tyFUNCTION,0,pv,&found);
	if( found )	{
		type = getType(a);
		chktype(type,tyFUNCTION);
//		if( arg != pv ) seterror("incorrect number of arguments");
	}
	return a;
}

//  install global name
int putSym(char *name, int type, int ref, int arg){
	int a, found;
	a = installGlobal(name,type,ref,arg,&found);
	if(found) seterror("redefine global name");
	return a;
}

void putvec(char *name, int size){
	putSym(name,tyVECTOR,DS,size);
	DS += size;
}

void putvar(char *name){
	putSym(name,tySCALAR,DS,1);
	DS++;
}
*/
void commit( int status ) {
	if ( status == 0 ) seterror("syntax");
}


// -------------- parser stack operator ----------

void ypush(int x){
	ysp++;
	if( ysp >= MAXYSTK )
		seterror("parser stack overflow");
	ystack[ysp] = x;
}

int ypop(void){
	int x;
	if( ysp <= 0 )
		seterror("parser stack underflow");
	x = ystack[ysp];
	ysp--;
	return x;
}

int ytos(void){
	return ystack[ysp];
}

// ------ bop operator stack -----------

void zpush(int op){
	zsp++;
	if( zsp >= MAXZSTK )
		seterror("operator stack overflow");
	zstack[zsp] = op;
}

int zpop(void){
	int x;
	if( zsp <= 0 )
		seterror("operator stack underflow");
	x = zstack[zsp];
	zsp--;
	return x;
}

int ztos(void){	return zstack[zsp];}

// precedence of binary operators
// = <> => > < <=                 (100)
// is in like between             (80)
// not-in not-like not-between    (80)
// and or                         (50)

// op: BOT * / - + = == and && or || not <> < <= > =>
//     ",", ";", "(", ")", "[", "]", "{", "}",
//	   "is", "in", "like", "between",
//	   "not in", "not like", "not between",

PRIVATE int prec[] = {
	0,0,0,0,0,100,0,50,0,50,0,0,100,100,100,100,100,
	0,0,0,0,0,0,0,0,
	80,80,80,80,
	80,80,80
	};

PRIVATE int precedence(int op){	return prec[op-tkBOT]; }

// [%e1 %e2 -- %bop] (bop e1 e2)
PRIVATE void dobop(int op){
	int e1, e2;
	e2 = ypop();
	e1 = cons(ypop(),list(e2));
	ypush(cons(newatom(OPER,op),e1));
}
//  read "do-bop-precedence.txt" for details
void zop(int op){
	if( precedence(op) > precedence(ztos()) ){
		zpush(op);
	}else{			// do tos
		dobop(zpop());
		zpush(op);
	}
}
// unwind operator stack
void zclean(void){
	int e;
	while( ztos() != tkBOT )
		dobop(zpop());
	e = zpop();			// throw away BOT
}

// ----------- action routines ------------

//  [ -- %var ]
void dovar(char *name){
	int a,type;
	int found;

	a = installGlobal(name,tySCALAR,0,0,&found);
	ypush(newatom(GNAME,a));
}

/*
// function call, n parameters
// [ MARK e1 .. en  -- %call]  (call name e1 .. en)
void docall(char *name){
    int a, ty, e, tmp;
	a = install(name);
	ty = getType(a);
	if( ty != tyFUNCTION )
		seterror("unknown function");
	e = NIL;
	while( ytos() != MARK )
		e = cons(ypop(),e);
	tmp = ypop();			// throw away MARK
	e = cons(newatom(GNAME, a), e);
	ypush(cons(newatom(OPER, tkCALL), e));
}
*/
void donum(char *numstr){
	ypush(newatom(NUM,atoi(numstr)));
}

void dostring(char *str){
	int p;
	p = freestr;
	strcpy(&strbuf[p],str);		// copy to strbuf[.]
	freestr += strlen(str) + 1;
	ypush(newatom(STRING,p));
}

void donull(void){
	dostring("null");
}

void donotnull(void){
	dostring("not null");
}

// [%ex -- %uop]
void douop(int op){
	ypush(cons(newatom(OPER, op), list(ypop())));
}

// [ MARK e1 e2 ... ]  =>  (list e1 e2 ...)
void dolist(void){
	int e, tmp;
	e = NIL;
	while( ytos() != MARK )
		e = cons(ypop(),e);
	tmp = ypop();			// throw away MARK
//	e = cons(newatom(GNAME, a), e);
	ypush(cons(newatom(OPER, tkLIST), e));
}

// [ var e1 e2 ] =>  (between var e1 e2)
void dobetween(void){
	int e, op;
	e = cons(ypop(),NIL);
	e = cons(ypop(),e);
	e = cons(ypop(),e);
	op = zpop();
	ypush(cons(newatom(OPER, op),e));
}


// end


